/*
 * This file is part of SpongeAPI, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.api.data.manipulator.immutable.entity;

import org.spongepowered.api.data.manipulator.ImmutableDataManipulator;
import org.spongepowered.api.data.manipulator.mutable.entity.AgeableData;
import org.spongepowered.api.data.value.immutable.ImmutableBoundedValue;
import org.spongepowered.api.data.value.immutable.ImmutableValue;
import org.spongepowered.api.entity.Entity;

/**
 * An {@link ImmutableDataManipulator} for handling the "age" of an
 * {@link Entity}.
 */
public interface ImmutableAgeableData extends ImmutableDataManipulator<ImmutableAgeableData, AgeableData> {

    /**
     * Gets the {@link ImmutableBoundedValue} for the "age" state.
     *
     * @return The immutable bounded value for the "age"
     */
    ImmutableBoundedValue<Integer> age();

    /**
     * Gets the {@link ImmutableValue} for whether the {@link #age()} is
     * considered to be a "baby". If the value is false, {@link #adult()}
     * should be {@code true} and vice versa.
     *
     * @return The immutable value for the "baby" state
     * @deprecated Due to this being added as the inverse of {@link #age()},
     *     this method is to be removed
     */
    @Deprecated
    ImmutableValue<Boolean> baby();

    /**
     * Gets the {@link ImmutableValue} for whether the {@link #age()} is
     * considered to be an "adult". If the value is false, {@link #baby()}
     * should be {@code true} and vice versa.
     *
     * @return The immutable value for the "adult" state
     */
    ImmutableValue<Boolean> adult();

}
